import { Identifiable, Identifier, Internship } from "@/data";
import { sampleCompanies, sampleStudent } from "@/provider/dummy";
import moment, { Moment } from "moment-timezone";
import { encapsulate, Nullable, OneOrMany } from "@/helpers";
import { SubmissionStatus } from "@/state/reducer/submission";
import { axios } from "@/api";
import { prepare, query } from "@/routing";
import { InternshipInfoDTO, submissionStateDtoTransformer } from "@/api/dto/internship-registration";
import { Transformer } from "@/serialization";
import { mentorDtoTransformer } from "@/api/dto/mentor";
import { internshipTypeDtoTransformer } from "@/api/dto/type";
import { studentDtoTransfer } from "@/api/dto/student";
import { programEntryDtoTransformer } from "@/api/dto/edition";

export type InternshipSubmission = Nullable<Internship> & {
    state: SubmissionStatus,
    changed: Moment | null,
}

const sampleInternship: InternshipSubmission = {
    id: "test",
    company: sampleCompanies[0],
    startDate: moment('2020-07-01'),
    endDate: moment('2020-08-31'),
    hours: 180,
    intern: sampleStudent,
    isAccepted: false,
    lengthInWeeks: 0,
    mentor: {
        email: "test@test.com",
        name: "Test",
        surname: "Testowy",
        phone: "+48 123 456 789"
    },
    office: sampleCompanies[0].offices[0],
    program: [
        { description: "Test" },
    ],
    type: {
        description: {
            pl: "Przykładowy typ",
            en: "Example type"
        },
        label: {
            pl: "Przykładowy",
            en: "Example"
        },
        requiresDeanApproval: true,
        requiresInsurance: false
    },
    state: "declined",
    changed: moment(),
}

const INTERNSHIP_MANAGEMENT_INDEX_ENDPOINT = "/management/internship";
const INTERNSHIP_MANAGEMENT_ENDPOINT       = "/management/internship/:id";
const INTERNSHIP_ACCEPT_ENDPOINT           = "/management/internship/accept/:id";
const INTERNSHIP_REJECT_ENDPOINT           = "/management/internship/reject/:id";

const internshipInfoDtoTransformer: Transformer<InternshipInfoDTO, InternshipSubmission> = {
    transform(subject: InternshipInfoDTO, context?: never): InternshipSubmission {
        return {
            changed: moment(subject.internshipRegistration.submissionDate),
            company: subject.internshipRegistration.company,
            startDate: moment(subject.internshipRegistration.start),
            endDate: moment(subject.internshipRegistration.end),
            hours: subject.internshipRegistration.declaredHours,
            id: subject.id,
            intern: subject.student && studentDtoTransfer.transform(subject.student),
            isAccepted: false,
            lengthInWeeks: 0,
            mentor: subject.internshipRegistration.mentor && mentorDtoTransformer.transform(subject.internshipRegistration.mentor),
            office: subject.internshipRegistration.branchAddress,
            program: (subject.internshipRegistration.subjects || []).map(subject => programEntryDtoTransformer.transform(subject.subject)),
            state: submissionStateDtoTransformer.transform(subject.internshipRegistration.state),
            type: subject.internshipRegistration.type && internshipTypeDtoTransformer.transform(subject.internshipRegistration.type),
        };
    },
    reverseTransform(subject: InternshipSubmission, context: undefined): InternshipInfoDTO {
        return {} as any;
    },
}

export async function all(edition: Identifiable): Promise<InternshipSubmission[]> {
    const result = await axios.get<InternshipInfoDTO[]>(query(INTERNSHIP_MANAGEMENT_INDEX_ENDPOINT, { EditionId: edition.id || "" }));

    return result.data.map(result => internshipInfoDtoTransformer.transform(result))
}

export async function get(id: Identifier): Promise<InternshipSubmission> {
    const result = await axios.get<InternshipInfoDTO>(prepare(INTERNSHIP_MANAGEMENT_ENDPOINT, { id }))

    return internshipInfoDtoTransformer.transform(result.data);
}

export async function accept(internship: OneOrMany<Internship>, comment?: string): Promise<void> {
    const internships = encapsulate(internship)

    await Promise.all(internships.map(internship => axios.put(prepare(INTERNSHIP_ACCEPT_ENDPOINT, { id: internship.id || ""}))))
}

export async function discard(internship: OneOrMany<Internship>, comment: string): Promise<void> {
    const internships = encapsulate(internship)

    await Promise.all(internships.map(internship => axios.put(prepare(INTERNSHIP_REJECT_ENDPOINT, { id: internship.id || ""}))))
}
